```@meta
EditURL = "https://codeberg.org/AdamWysokinski/NeuroAnalyzer.jl/src/branch/main/docs/src/index.md"
```

![](assets/neuroanalyzer.png)
# NeuroAnalyzer.jl documentation

This documentation has been generated using [Documenter.jl](https://juliadocs.github.io/Documenter.jl/stable/).

## NeuroAnalyzer

```@docs
NeuroAnalyzer.na_info
NeuroAnalyzer.na_plugins_install
NeuroAnalyzer.na_plugins_list
NeuroAnalyzer.na_plugins_reload
NeuroAnalyzer.na_plugins_remove
NeuroAnalyzer.na_plugins_update
NeuroAnalyzer.na_set_prefs
NeuroAnalyzer.na_set_progress_bar
NeuroAnalyzer.na_set_use_cuda
NeuroAnalyzer.na_set_verbose
NeuroAnalyzer.na_version
```

## Utils

```@docs
NeuroAnalyzer.add_component
NeuroAnalyzer.add_component!
NeuroAnalyzer.add_note
NeuroAnalyzer.add_note!
NeuroAnalyzer.apply
NeuroAnalyzer.areduce
NeuroAnalyzer.arr2mat
NeuroAnalyzer.band_frq
NeuroAnalyzer.Base.size
NeuroAnalyzer.cextrema
NeuroAnalyzer.channel_cluster
NeuroAnalyzer.channel_pick
NeuroAnalyzer.chtypes
NeuroAnalyzer.cmax
NeuroAnalyzer.cmin
NeuroAnalyzer.component_type
NeuroAnalyzer.cums
NeuroAnalyzer.datatype
NeuroAnalyzer.delete_component
NeuroAnalyzer.delete_component!
NeuroAnalyzer.delete_note
NeuroAnalyzer.delete_note!
NeuroAnalyzer.delmean
NeuroAnalyzer.describe
NeuroAnalyzer.detector_labels
NeuroAnalyzer.epoch_len
NeuroAnalyzer.extract_component
NeuroAnalyzer.f2t
NeuroAnalyzer.fft0
NeuroAnalyzer.fft2
NeuroAnalyzer.fft_transform
NeuroAnalyzer.findpeaks
NeuroAnalyzer.f_nearest
NeuroAnalyzer.freqs
NeuroAnalyzer.generate_csine
NeuroAnalyzer.generate_gaussian
NeuroAnalyzer.generate_morlet
NeuroAnalyzer.generate_morlet_fwhm
NeuroAnalyzer.generate_noise
NeuroAnalyzer.generate_sinc
NeuroAnalyzer.generate_sine
NeuroAnalyzer.generate_square
NeuroAnalyzer.generate_triangle
NeuroAnalyzer.generate_window
NeuroAnalyzer.gradient
NeuroAnalyzer.history
NeuroAnalyzer.hz2rads
NeuroAnalyzer.ifft0
NeuroAnalyzer.info
NeuroAnalyzer.l1
NeuroAnalyzer.l2
NeuroAnalyzer.labels
NeuroAnalyzer.linspace
NeuroAnalyzer.list_component
NeuroAnalyzer.logspace
NeuroAnalyzer.make_table
NeuroAnalyzer.maxat
NeuroAnalyzer.minat
NeuroAnalyzer.m_norm
NeuroAnalyzer.m_pad0
NeuroAnalyzer.m_sort
NeuroAnalyzer.m_sortperm
NeuroAnalyzer.nchannels
NeuroAnalyzer.nepochs
NeuroAnalyzer.nextpow2
NeuroAnalyzer.optode_labels
NeuroAnalyzer.pad0
NeuroAnalyzer.pad2
NeuroAnalyzer.padm
NeuroAnalyzer.paired_labels
NeuroAnalyzer.perm_cmp
NeuroAnalyzer.phases
NeuroAnalyzer.rads2hz
NeuroAnalyzer.rename_component
NeuroAnalyzer.rename_component!
NeuroAnalyzer.reset_components
NeuroAnalyzer.reset_components!
NeuroAnalyzer.rfft0
NeuroAnalyzer.rfft2
NeuroAnalyzer.s2t
NeuroAnalyzer.signal_len
NeuroAnalyzer.source_labels
NeuroAnalyzer.sr
NeuroAnalyzer.t2f
NeuroAnalyzer.t2s
NeuroAnalyzer.tavg
NeuroAnalyzer.to_df
NeuroAnalyzer.tuple_order
NeuroAnalyzer.vec2mat
NeuroAnalyzer.view_header
NeuroAnalyzer.view_note
NeuroAnalyzer.vreduce
NeuroAnalyzer.vsearch
NeuroAnalyzer.vsplit
```

## IO

```@docs
NeuroAnalyzer.export_csv
NeuroAnalyzer.export_locs
NeuroAnalyzer.export_markers
NeuroAnalyzer.import_alice4
NeuroAnalyzer.import_bdf
NeuroAnalyzer.import_bv
NeuroAnalyzer.import_cnt
NeuroAnalyzer.import_csv
NeuroAnalyzer.import_dat
NeuroAnalyzer.import_digitrack
NeuroAnalyzer.import_duomag
NeuroAnalyzer.import_edf
NeuroAnalyzer.import_edf_annotations
NeuroAnalyzer.import_fiff
NeuroAnalyzer.import_gdf
NeuroAnalyzer.import_locs
NeuroAnalyzer.import_locs_asc
NeuroAnalyzer.import_locs_ced
NeuroAnalyzer.import_locs_csd
NeuroAnalyzer.import_locs_dat
NeuroAnalyzer.import_locs_elc
NeuroAnalyzer.import_locs_geo
NeuroAnalyzer.import_locs_locs
NeuroAnalyzer.import_locs_mat
NeuroAnalyzer.import_locs_sfp
NeuroAnalyzer.import_locs_tsv
NeuroAnalyzer.import_locs_txt
NeuroAnalyzer.import_montage
NeuroAnalyzer.import_ncs
NeuroAnalyzer.import_nirs
NeuroAnalyzer.import_nirx
NeuroAnalyzer.import_npy
NeuroAnalyzer.import_nwb
NeuroAnalyzer.import_recording
NeuroAnalyzer.import_set
NeuroAnalyzer.import_snirf
NeuroAnalyzer.import_thymatron
NeuroAnalyzer.import_xdf
NeuroAnalyzer.load
NeuroAnalyzer.load_locs
NeuroAnalyzer.load_locs!
NeuroAnalyzer.load_study
NeuroAnalyzer.save
NeuroAnalyzer.save_study
```

## Edit

```@docs
NeuroAnalyzer.add_channel
NeuroAnalyzer.add_channel!
NeuroAnalyzer.add_labels
NeuroAnalyzer.add_labels!
NeuroAnalyzer.add_marker
NeuroAnalyzer.add_marker!
NeuroAnalyzer.channel2marker
NeuroAnalyzer.channel2marker!
NeuroAnalyzer.chop
NeuroAnalyzer.chop!
NeuroAnalyzer.create_data
NeuroAnalyzer.create_data!
NeuroAnalyzer.create_object
NeuroAnalyzer.create_time
NeuroAnalyzer.create_time!
NeuroAnalyzer.delete_channel
NeuroAnalyzer.delete_channel!
NeuroAnalyzer.delete_epoch
NeuroAnalyzer.delete_epoch!
NeuroAnalyzer.delete_marker
NeuroAnalyzer.delete_marker!
NeuroAnalyzer.delete_optode
NeuroAnalyzer.delete_optode!
NeuroAnalyzer.detect_bad
NeuroAnalyzer.edit_channel
NeuroAnalyzer.edit_channel!
NeuroAnalyzer.edit_marker
NeuroAnalyzer.edit_marker!
NeuroAnalyzer.epoch
NeuroAnalyzer.epoch!
NeuroAnalyzer.epoch_ts
NeuroAnalyzer.epoch_ts!
NeuroAnalyzer.extract_channel
NeuroAnalyzer.extract_data
NeuroAnalyzer.extract_epoch
NeuroAnalyzer.extract_epoch!
NeuroAnalyzer.extract_eptime
NeuroAnalyzer.extract_time
NeuroAnalyzer.get_channel
NeuroAnalyzer.get_channel_bytype
NeuroAnalyzer.get_channel_bywl
NeuroAnalyzer.get_channel_type
NeuroAnalyzer.join
NeuroAnalyzer.join!
NeuroAnalyzer.keep_channel
NeuroAnalyzer.keep_channel!
NeuroAnalyzer.keep_channel_type
NeuroAnalyzer.keep_channel_type!
NeuroAnalyzer.keep_epoch
NeuroAnalyzer.keep_epoch!
NeuroAnalyzer.reflect
NeuroAnalyzer.reflect!
NeuroAnalyzer.rename_channel
NeuroAnalyzer.rename_channel!
NeuroAnalyzer.replace_channel
NeuroAnalyzer.replace_channel!
NeuroAnalyzer.set_channel_type
NeuroAnalyzer.set_channel_type!
NeuroAnalyzer.signal_channels
NeuroAnalyzer.trim
NeuroAnalyzer.trim!
NeuroAnalyzer.vch
NeuroAnalyzer.view_marker
```

## Process

```@docs
NeuroAnalyzer.add_signal
NeuroAnalyzer.add_signal!
NeuroAnalyzer.average
NeuroAnalyzer.average!
NeuroAnalyzer.bpsplit
NeuroAnalyzer.cbp
NeuroAnalyzer.cbp!
NeuroAnalyzer.ch_zero
NeuroAnalyzer.ch_zero!
NeuroAnalyzer.csd
NeuroAnalyzer.csd!
NeuroAnalyzer.cw_trans
NeuroAnalyzer.denoise_fft
NeuroAnalyzer.denoise_fft!
NeuroAnalyzer.denoise_wavelet
NeuroAnalyzer.denoise_wavelet!
NeuroAnalyzer.denoise_wien
NeuroAnalyzer.denoise_wien!
NeuroAnalyzer.derivative
NeuroAnalyzer.derivative!
NeuroAnalyzer.detect_powerline
NeuroAnalyzer.detrend
NeuroAnalyzer.detrend!
NeuroAnalyzer.downsample
NeuroAnalyzer.downsample!
NeuroAnalyzer.dw_trans
NeuroAnalyzer.dwtsplit
NeuroAnalyzer.edit_montage
NeuroAnalyzer.erp
NeuroAnalyzer.erp!
NeuroAnalyzer.fconv
NeuroAnalyzer.fconv!
NeuroAnalyzer.filter
NeuroAnalyzer.filter!
NeuroAnalyzer.filter_apply
NeuroAnalyzer.filter_create
NeuroAnalyzer.filter_g
NeuroAnalyzer.filter_g!
NeuroAnalyzer.filter_mavg
NeuroAnalyzer.filter_mavg!
NeuroAnalyzer.filter_mmed
NeuroAnalyzer.filter_mmed!
NeuroAnalyzer.filter_poly
NeuroAnalyzer.filter_poly!
NeuroAnalyzer.filter_sg
NeuroAnalyzer.filter_sg!
NeuroAnalyzer.gh
NeuroAnalyzer.ica_decompose
NeuroAnalyzer.ica_reconstruct
NeuroAnalyzer.ica_reconstruct!
NeuroAnalyzer.ica_remove
NeuroAnalyzer.ica_remove!
NeuroAnalyzer.icw_trans
NeuroAnalyzer.idw_trans
NeuroAnalyzer.intensity2od
NeuroAnalyzer.intensity2od!
NeuroAnalyzer.invert_polarity
NeuroAnalyzer.invert_polarity!
NeuroAnalyzer.lrinterpolate_channel
NeuroAnalyzer.lrinterpolate_channel!
NeuroAnalyzer.normalize
NeuroAnalyzer.normalize!
NeuroAnalyzer.normalize_gauss
NeuroAnalyzer.normalize_invroot
NeuroAnalyzer.normalize_log
NeuroAnalyzer.normalize_log10
NeuroAnalyzer.normalize_minmax
NeuroAnalyzer.normalize_n
NeuroAnalyzer.normalize_neg
NeuroAnalyzer.normalize_neglog
NeuroAnalyzer.normalize_neglog10
NeuroAnalyzer.normalize_perc
NeuroAnalyzer.normalize_pos
NeuroAnalyzer.normalize_softmax
NeuroAnalyzer.normalize_zscore
NeuroAnalyzer.normpower
NeuroAnalyzer.normpower!
NeuroAnalyzer.npl
NeuroAnalyzer.npl!
NeuroAnalyzer.od2conc
NeuroAnalyzer.od2conc!
NeuroAnalyzer.pca_decompose
NeuroAnalyzer.pca_reconstruct
NeuroAnalyzer.pca_reconstruct!
NeuroAnalyzer.plinterpolate
NeuroAnalyzer.plinterpolate_channel
NeuroAnalyzer.plinterpolate_channel!
NeuroAnalyzer.reference_a
NeuroAnalyzer.reference_a!
NeuroAnalyzer.reference_avg
NeuroAnalyzer.reference_avg!
NeuroAnalyzer.reference_ce
NeuroAnalyzer.reference_ce!
NeuroAnalyzer.reference_custom
NeuroAnalyzer.reference_custom!
NeuroAnalyzer.reference_m
NeuroAnalyzer.reference_m!
NeuroAnalyzer.reference_plap
NeuroAnalyzer.reference_plap!
NeuroAnalyzer.remove_dc
NeuroAnalyzer.remove_dc!
NeuroAnalyzer.remove_pops
NeuroAnalyzer.remove_pops!
NeuroAnalyzer.remove_powerline
NeuroAnalyzer.remove_powerline!
NeuroAnalyzer.resample
NeuroAnalyzer.resample!
NeuroAnalyzer.scale
NeuroAnalyzer.scale!
NeuroAnalyzer.sort_epochs
NeuroAnalyzer.sort_epochs!
NeuroAnalyzer.standardize
NeuroAnalyzer.standardize!
NeuroAnalyzer.taper
NeuroAnalyzer.taper!
NeuroAnalyzer.tconv
NeuroAnalyzer.upsample
NeuroAnalyzer.upsample!
NeuroAnalyzer.wbp
NeuroAnalyzer.wbp!
```

## Locs

```@docs
NeuroAnalyzer.add_locs
NeuroAnalyzer.add_locs!
NeuroAnalyzer.cart2pol
NeuroAnalyzer.cart2sph
NeuroAnalyzer.edit_locs
NeuroAnalyzer.edit_locs!
NeuroAnalyzer.locs_cart2pol
NeuroAnalyzer.locs_cart2pol!
NeuroAnalyzer.locs_cart2sph
NeuroAnalyzer.locs_cart2sph!
NeuroAnalyzer.locs_center
NeuroAnalyzer.locs_center!
NeuroAnalyzer.locs_details
NeuroAnalyzer.locs_flipx
NeuroAnalyzer.locs_flipx!
NeuroAnalyzer.locs_flipy
NeuroAnalyzer.locs_flipy!
NeuroAnalyzer.locs_flipz
NeuroAnalyzer.locs_flipz!
NeuroAnalyzer.locs_generate
NeuroAnalyzer.locs_generate!
NeuroAnalyzer.locs_normalize
NeuroAnalyzer.locs_normalize!
NeuroAnalyzer.locs_pol2cart
NeuroAnalyzer.locs_pol2cart!
NeuroAnalyzer.locs_pol2sph
NeuroAnalyzer.locs_pol2sph!
NeuroAnalyzer.locs_rotx
NeuroAnalyzer.locs_rotx!
NeuroAnalyzer.locs_roty
NeuroAnalyzer.locs_roty!
NeuroAnalyzer.locs_rotz
NeuroAnalyzer.locs_rotz!
NeuroAnalyzer.locs_scale
NeuroAnalyzer.locs_scale!
NeuroAnalyzer.locs_sph2cart
NeuroAnalyzer.locs_sph2cart!
NeuroAnalyzer.locs_sph2pol
NeuroAnalyzer.locs_sph2pol!
NeuroAnalyzer.locs_swapxy
NeuroAnalyzer.locs_swapxy!
NeuroAnalyzer.pol2cart
NeuroAnalyzer.pol2sph
NeuroAnalyzer.sph2cart
NeuroAnalyzer.sph2pol
```

## Analyze

### Analyze: amplitude

```@docs
NeuroAnalyzer.amp
NeuroAnalyzer.ampdiff
NeuroAnalyzer.rmse
NeuroAnalyzer.snr
NeuroAnalyzer.snr2
```

### Analyze: frequency

```@docs
NeuroAnalyzer.band_asymmetry
NeuroAnalyzer.band_mpower
NeuroAnalyzer.band_power
NeuroAnalyzer.coherence
NeuroAnalyzer.cpsd
NeuroAnalyzer.cwtspectrogram
NeuroAnalyzer.erop
NeuroAnalyzer.eros
NeuroAnalyzer.frqinst
NeuroAnalyzer.ghspectrogram
NeuroAnalyzer.hjorth
NeuroAnalyzer.hspectrum
NeuroAnalyzer.mwpsd
NeuroAnalyzer.mwspectrogram
NeuroAnalyzer.peak_frq
NeuroAnalyzer.psd
NeuroAnalyzer.psd_rel
NeuroAnalyzer.psd_slope
NeuroAnalyzer.sef
NeuroAnalyzer.spectrogram
NeuroAnalyzer.spectrum
NeuroAnalyzer.total_power
```

### Analyze: time

```@docs
NeuroAnalyzer.acor
NeuroAnalyzer.acov
NeuroAnalyzer.axc2frq
NeuroAnalyzer.pacor
NeuroAnalyzer.xcor
NeuroAnalyzer.xcov
```

### Analyze: phase

```@docs
NeuroAnalyzer.cph
NeuroAnalyzer.ispc
NeuroAnalyzer.itpc
NeuroAnalyzer.itpc_spec
NeuroAnalyzer.phdiff
NeuroAnalyzer.phsd
NeuroAnalyzer.pli
```

### Analyze: ERP

```@docs
NeuroAnalyzer.amp_at
NeuroAnalyzer.avgamp_at
NeuroAnalyzer.erp_peaks
NeuroAnalyzer.maxamp_at
NeuroAnalyzer.minamp_at
```

### Analyze: MEP

```@docs
NeuroAnalyzer.mep_peaks
```

### Analyze: misc

```@docs
NeuroAnalyzer.corm
NeuroAnalyzer.covm
NeuroAnalyzer.diss
NeuroAnalyzer.entropy
NeuroAnalyzer.env_cor
NeuroAnalyzer.env_lo
NeuroAnalyzer.env_up
NeuroAnalyzer.ged
NeuroAnalyzer.gfp
NeuroAnalyzer.gfp_norm
NeuroAnalyzer.henv
NeuroAnalyzer.henv_lo
NeuroAnalyzer.henv_mean
NeuroAnalyzer.henv_median
NeuroAnalyzer.henv_up
NeuroAnalyzer.hrv_analyze
NeuroAnalyzer.hrv_detect
NeuroAnalyzer.mutual_information
NeuroAnalyzer.negentropy
NeuroAnalyzer.penv
NeuroAnalyzer.penv_mean
NeuroAnalyzer.penv_median
NeuroAnalyzer.senv
NeuroAnalyzer.senv_mean
NeuroAnalyzer.senv_median
NeuroAnalyzer.stationarity
NeuroAnalyzer.stationarity_hilbert
NeuroAnalyzer.stationarity_mean
NeuroAnalyzer.stationarity_var
NeuroAnalyzer.tenv
NeuroAnalyzer.tenv_mean
NeuroAnalyzer.tenv_median
NeuroAnalyzer.tkeo
```

## Plot

```@docs
NeuroAnalyzer.add_locs
NeuroAnalyzer.add_to_canvas
NeuroAnalyzer.add_topmargin_canvas
NeuroAnalyzer.plot
NeuroAnalyzer.plot2canvas
NeuroAnalyzer.plot_2signals
NeuroAnalyzer.plot_bar
NeuroAnalyzer.plot_box
NeuroAnalyzer.plot_ci
NeuroAnalyzer.plot_coherence
NeuroAnalyzer.plot_coherence_avg
NeuroAnalyzer.plot_coherence_butterfly
NeuroAnalyzer.plot_compose
NeuroAnalyzer.plot_connections
NeuroAnalyzer.plot_dipole2d
NeuroAnalyzer.plot_dipole3d
NeuroAnalyzer.plot_dots
NeuroAnalyzer.plot_empty
NeuroAnalyzer.plot_erop
NeuroAnalyzer.plot_eros
NeuroAnalyzer.plot_erp
NeuroAnalyzer.plot_erp_avg
NeuroAnalyzer.plot_erp_butterfly
NeuroAnalyzer.plot_erp_stack
NeuroAnalyzer.plot_erp_topo
NeuroAnalyzer.plot_filter_response
NeuroAnalyzer.plot_histogram
NeuroAnalyzer.plot_icatopo
NeuroAnalyzer.plot_line
NeuroAnalyzer.plot_locs
NeuroAnalyzer.plot_locs3d
NeuroAnalyzer.plot_locs_nirs
NeuroAnalyzer.plot_matrix
NeuroAnalyzer.plot_mep
NeuroAnalyzer.plot_mep_avg
NeuroAnalyzer.plot_mep_butterfly
NeuroAnalyzer.plot_mep_stack
NeuroAnalyzer.plot_paired
NeuroAnalyzer.plot_phsd
NeuroAnalyzer.plot_phsd_3d
NeuroAnalyzer.plot_phsd_avg
NeuroAnalyzer.plot_phsd_butterfly
NeuroAnalyzer.plot_phsd_topo
NeuroAnalyzer.plot_polar
NeuroAnalyzer.plot_psd
NeuroAnalyzer.plot_psd_3d
NeuroAnalyzer.plot_psd_avg
NeuroAnalyzer.plot_psd_butterfly
NeuroAnalyzer.plot_psd_topo
NeuroAnalyzer.plot_save
NeuroAnalyzer.plot_signal
NeuroAnalyzer.plot_signal_avg
NeuroAnalyzer.plot_signal_butterfly
NeuroAnalyzer.plot_spectrogram
NeuroAnalyzer.plot_topo
NeuroAnalyzer.plot_violin
NeuroAnalyzer.plot_weights
NeuroAnalyzer.plot_xac
NeuroAnalyzer.resize_canvas
```

## GUI

```@docs
NeuroAnalyzer.iedit
NeuroAnalyzer.iedit_ch
NeuroAnalyzer.iedit_cont
NeuroAnalyzer.iedit_ep
NeuroAnalyzer.iplot
NeuroAnalyzer.iplot_cont
NeuroAnalyzer.iplot_ep
NeuroAnalyzer.iplot_icatopo
NeuroAnalyzer.iplot_locs3d
NeuroAnalyzer.ipsd
NeuroAnalyzer.ipsd_cont
NeuroAnalyzer.ipsd_ep
NeuroAnalyzer.iselect_seg
NeuroAnalyzer.iselect_ts
NeuroAnalyzer.ispectrogram
NeuroAnalyzer.ispectrogram_cont
NeuroAnalyzer.ispectrogram_ep
NeuroAnalyzer.itopo
NeuroAnalyzer.iview
NeuroAnalyzer.iview_cont
NeuroAnalyzer.iview_ep
NeuroAnalyzer.iview_plot
```

## Statistics

```@docs
NeuroAnalyzer.binom_prob
NeuroAnalyzer.binom_stat
NeuroAnalyzer.bootstrap_ci
NeuroAnalyzer.bootstrap_stat
NeuroAnalyzer.channel_stats
NeuroAnalyzer.ci2z
NeuroAnalyzer.ci_median
NeuroAnalyzer.ci_r
NeuroAnalyzer.cmp_stat
NeuroAnalyzer.cmp_test
NeuroAnalyzer.cor_test
NeuroAnalyzer.count_thresh
NeuroAnalyzer.crit_z
NeuroAnalyzer.cvar
NeuroAnalyzer.cvar_mean
NeuroAnalyzer.cvar_median
NeuroAnalyzer.dprime
NeuroAnalyzer.dranks
NeuroAnalyzer.effsize
NeuroAnalyzer.epoch_stats
NeuroAnalyzer.f1
NeuroAnalyzer.friedman
NeuroAnalyzer.grubbs
NeuroAnalyzer.hildebrand_rule
NeuroAnalyzer.infcrit
NeuroAnalyzer.jaccard_similarity
NeuroAnalyzer.k_categories
NeuroAnalyzer.linreg
NeuroAnalyzer.mcc
NeuroAnalyzer.mdiff
NeuroAnalyzer.meanc
NeuroAnalyzer.meang
NeuroAnalyzer.meanh
NeuroAnalyzer.meanw
NeuroAnalyzer.moe
NeuroAnalyzer.msci95
NeuroAnalyzer.mscr
NeuroAnalyzer.norminv
NeuroAnalyzer.outlier_detect
NeuroAnalyzer.power_c1g
NeuroAnalyzer.power_c2g
NeuroAnalyzer.power_p1g
NeuroAnalyzer.power_p2g
NeuroAnalyzer.prank
NeuroAnalyzer.pred_int
NeuroAnalyzer.res_norm
NeuroAnalyzer.rng
NeuroAnalyzer.r_test
NeuroAnalyzer.se
NeuroAnalyzer.seg_extract
NeuroAnalyzer.seg_mean
NeuroAnalyzer.sem_diff
NeuroAnalyzer.size_c1diff
NeuroAnalyzer.size_c1g
NeuroAnalyzer.size_c2g
NeuroAnalyzer.size_p1diff
NeuroAnalyzer.size_p1g
NeuroAnalyzer.size_p2g
NeuroAnalyzer.spec_seg
NeuroAnalyzer.summary
NeuroAnalyzer.vartest
NeuroAnalyzer.z2pow
NeuroAnalyzer.z_score
```

## Study

```@docs
NeuroAnalyzer.create_study
NeuroAnalyzer.epoch_len
NeuroAnalyzer.nchannels
NeuroAnalyzer.nepochs
NeuroAnalyzer.obj_n
NeuroAnalyzer.sr
```

## NeuroRecorder

```@docs
NeuroAnalyzer.ftt
NeuroAnalyzer.iftt
```

## NeuroStim

```@docs
NeuroAnalyzer.ect_charge
NeuroAnalyzer.tes_dose
NeuroAnalyzer.tes_model
NeuroAnalyzer.tes_protocol
```

## NeuroTester

```@docs
```
